#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <typeinfo>

#include <gtk/gtk.h>
#include <glib.h>

#include <gPODEnv.h>

typedef struct PODData {
   int PODNumber;
   GtkWidget *TypeWidget; /* The POD Type Widget */
   GtkWidget *PODName; /* The Text Entry Widget */
   GtkWidget *TestButton; /* The Test Button Widget */
   GtkWidget *InfoButton; /* The Info Button Widget */
   POD *ThisPOD;
};

gPODEnv *AllPODs = NULL;
int Displayed = 0;
PODData PODEditData[MAXPODS];

gint destroyapp(GtkWidget *widget, gpointer gdata)
{
   g_print("Quitting...\n");
   gtk_main_quit();
   return(FALSE);
}


static char* MakeUTF8(const char *latin1)
{
   const char *p;
   GString *utf8;

   utf8 = g_string_new (NULL);

   for (p = latin1; *p != '\0'; p++) g_string_append_unichar (utf8, (gunichar) *p);
   return g_string_free (utf8, FALSE);
}

void TestButtonPressed(GtkWidget *widget, gpointer TempData)
{
   PODData MyPODData;
   POD *MyPOD;
   int TempInt;

   MyPODData = * (PODData *)(TempData);
   MyPOD = MyPODData.ThisPOD;
   TempInt = MyPODData.PODNumber;

   printf("Test button for %d pressed\n", TempInt);
   MyPOD->Test();
}

static void TextEntryModified(GtkWidget *Widget, void *TempData)
{
   const gchar *EntryText;
   char TempText[100];
   int TempInt;
   PODData MyPODData;
   POD *MyPOD;

   MyPODData = * (PODData *)(TempData);
   TempInt = MyPODData.PODNumber;
   MyPOD = MyPODData.ThisPOD;
   EntryText = gtk_entry_get_text (GTK_ENTRY (Widget));
   printf("Text for %d changed to %s\n", TempInt, EntryText);
//   assert (MyPOD != NULL);
   if (MyPOD != NULL)
   {
      sprintf(TempText, "%s", EntryText);
      printf("Setting name\n");
      MyPOD->SetName(TempText);
   }
}

int Table_Add_Widget(GtkWidget *Table, GtkWidget *Widget, int Column, int Row)
{
   gtk_table_attach(GTK_TABLE(Table), Widget, Column, Column + 1, Row, Row + 1, (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL), (GtkAttachOptions)(GTK_EXPAND | GTK_SHRINK | GTK_FILL), 0, 0);
   gtk_widget_show(Widget);
   return(0);
}

/* This function assumes that the label will take up a single cell */
GtkWidget *Table_Add_Label(GtkWidget *Table, char *LabelText, int Column, int Row)
{
   GtkWidget *Label;

   Label = gtk_label_new(LabelText);
   Table_Add_Widget(Table, Label, Column, Row);
//   gtk_widget_show(Label);
   return(Label);
}
int SetupPODs()
{
   int Counter;
   char PODTypeText[100];
   char PODNameText[100];
   int PODType;
   GtkWidget *TempWidget;
   POD *CurrentPOD;
   gboolean ValidPOD;

   if (AllPODs == NULL) AllPODs = new gPODEnv(POD_ENV_OPTION_TEXT_STATUS | POD_ENV_OPTION_GUI_STATUS | POD_ENV_OPTION_DONT_INIT);
   else AllPODs->ResetNumberOfPODs();
//   AllPODs->DisplayProgressBar(1, 0.0);
   AllPODs->DisplayStatus(1, 0);
   for (Counter = 0 ; Counter < MAXPODS ; Counter ++)
   {
//      AllPODs->DisplayProgressBar(1, (Counter + 1.0) / (MAXPODS + 1.0));
      AllPODs->DisplayStatus(1, Counter);

      AllPODs->RefreshPOD(Counter);
      TempWidget = PODEditData[Counter].TypeWidget;
      if (Displayed)
      {
         CurrentPOD = AllPODs->GetPOD(Counter);
         PODEditData[Counter].ThisPOD = CurrentPOD;

         if (CurrentPOD != NULL) PODType = CurrentPOD->GetType(PODTypeText);
         else PODType = POD_TYPE_NONE;

         if ((CurrentPOD != NULL) && (PODType != POD_TYPE_NONE))
         {
            CurrentPOD->GetName(PODNameText);
            ValidPOD = TRUE;
         }
         else
         {
            sprintf(PODTypeText, "No POD Connected");
            sprintf(PODNameText, "NONE");
            ValidPOD = FALSE;
         }

         gtk_label_set_text(GTK_LABEL(PODEditData[Counter].TypeWidget), MakeUTF8(PODTypeText));

         gtk_entry_set_text(GTK_ENTRY(PODEditData[Counter].PODName), MakeUTF8(PODNameText));
         gtk_entry_set_editable(GTK_ENTRY(PODEditData[Counter].PODName), ValidPOD);
         gtk_widget_set_sensitive((GtkWidget *)(PODEditData[Counter].PODName), ValidPOD);
         gtk_widget_set_sensitive((GtkWidget *)(PODEditData[Counter].TestButton), ValidPOD);
         while (g_main_iteration (FALSE)); /* Display any gtk updates */
//         usleep(200000);
      }
      while (g_main_iteration (FALSE)); /* Display any gtk updates */
   }
   AllPODs->DisplayStatus(1, MAXPODS);
   AllPODs->DisplayStatus(0, 0);
//   AllPODs->DisplayProgressBar(1, 1.0); /* Set ProgressBar to Done */
//   AllPODs->DisplayProgressBar(0, 1.0); /* Turn off ProgressBar */
   return(0);
}

void RefreshButtonPressed(GtkWidget *widget, gpointer TempData)
{
   static char Running = 0; /* protect against multiple entry into the function */
   if (Running) return;
   Running ++;
   SetupPODs();
   Running --;
}

int EditPODs()
{
   GtkWidget *window;
   GtkWidget *Table;
   GtkWidget *RefreshButton;
   GtkWidget *QuitButton;
   char TempText[100];
   POD *CurrentPOD;
   int Counter;
   int PODType;
   char PODTypeText[100];
   char PODNameText[100];
   gboolean ValidPOD;
   GtkWidget *TempWidget;

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

   Table = gtk_table_new(MAXPODS + 2, 5, FALSE);
   gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(destroyapp), NULL);


//   Table_Add_Label(Table, "POD#", 0, 0);

   TempWidget = gtk_label_new("<big>POD#</big>");
   gtk_label_set_use_markup(GTK_LABEL(TempWidget), TRUE);
   Table_Add_Widget(Table, TempWidget, 0, 0);

   Table_Add_Label(Table, "Type", 1, 0);
   Table_Add_Label(Table, "Name", 2, 0);
   Table_Add_Label(Table, "Test", 3, 0);
   Table_Add_Label(Table, "Info", 4, 0);

   for (Counter = 0 ; Counter < MAXPODS ; Counter ++)
   {
      PODEditData[Counter].PODNumber = Counter;

      sprintf(TempText, "%d", Counter);
      Table_Add_Label(Table, TempText, 0, Counter + 1);

      CurrentPOD = NULL;

      if (CurrentPOD != NULL) PODType = CurrentPOD->GetType(PODTypeText);
      else PODType = POD_TYPE_NONE;

      if ((CurrentPOD != NULL) && (PODType != POD_TYPE_NONE))
      {
         CurrentPOD->GetName(PODNameText);
         ValidPOD = TRUE;
      }
      else
      {
         sprintf(PODTypeText, "No POD Connected");
         sprintf(PODNameText, "NONE");
         ValidPOD = FALSE;
      }
      PODEditData[Counter].TypeWidget = Table_Add_Label(Table, PODTypeText, 1, Counter + 1);

      PODEditData[Counter].PODName = gtk_entry_new();
      gtk_entry_set_editable(GTK_ENTRY(PODEditData[Counter].PODName), ValidPOD);

      gtk_entry_set_text(GTK_ENTRY(PODEditData[Counter].PODName), MakeUTF8(PODNameText));
      gtk_entry_set_max_length(GTK_ENTRY(PODEditData[Counter].PODName), POD_SIZE_NAME);
      gtk_signal_connect (GTK_OBJECT(PODEditData[Counter].PODName), "activate", GTK_SIGNAL_FUNC(TextEntryModified), (gpointer)(&PODEditData[Counter]));
      Table_Add_Widget(Table, PODEditData[Counter].PODName, 2, Counter + 1);

      PODEditData[Counter].TestButton = gtk_button_new_with_label(MakeUTF8("Test"));
      gtk_signal_connect(GTK_OBJECT(PODEditData[Counter].TestButton), "clicked", GTK_SIGNAL_FUNC(TestButtonPressed), (gpointer)(&PODEditData[Counter]));
      Table_Add_Widget(Table, PODEditData[Counter].TestButton, 3, Counter + 1);
   }

   RefreshButton = gtk_button_new_with_label(MakeUTF8("Refresh PODs"));
   gtk_signal_connect(GTK_OBJECT(RefreshButton), "clicked", GTK_SIGNAL_FUNC(RefreshButtonPressed), (gpointer)(NULL));
   Table_Add_Widget(Table, RefreshButton, 0, MAXPODS + 2);
   gtk_widget_show(RefreshButton);

   QuitButton = gtk_button_new_with_label(MakeUTF8("Quit"));
   gtk_signal_connect(GTK_OBJECT(QuitButton), "clicked", GTK_SIGNAL_FUNC(destroyapp), (gpointer)(NULL));
   Table_Add_Widget(Table, QuitButton, 3, MAXPODS + 2);
   gtk_widget_show(QuitButton);

   gtk_container_add(GTK_CONTAINER(window), Table);
   gtk_container_border_width (GTK_CONTAINER (window), 15);
   gtk_window_set_title(GTK_WINDOW(window), MakeUTF8("POD Editor"));

   gtk_widget_show_all(window);
   while (g_main_iteration (FALSE)); /* display any gtk updates */
   Displayed = 1;

   return(0);
}

int main(int argc, char *argv[])
{
   int Counter;

//   gtk_init(&argc, &argv);
   gtk_init(&argc, &argv);
//   gtk_init(NULL, NULL);

   for (Counter = 0 ; Counter < MAXPODS ; Counter ++)
   {
      PODEditData[Counter].ThisPOD = NULL;
   }

   EditPODs();

   SetupPODs();

   gtk_main();

   return(0);
}
